function AuctionExtension.EGUI.AttachDragFrame(parent, fTitle, variable, fLeft)
	local bindedFrame = parent
	if fTitle then
		bindedFrame = parent:GetBorder()
	end

	local frame = UI.CreateFrame("Frame", "", bindedFrame)
	frame:SetPoint("TOPLEFT", bindedFrame, "TOPLEFT", 0, 0)
	if fTitle then
		frame:SetPoint("BOTTOMRIGHT", bindedFrame, "TOPRIGHT", 0, parent:GetTop() - bindedFrame:GetTop())
	else
		frame:SetPoint("BOTTOMRIGHT", bindedFrame, "BOTTOMRIGHT", 0, 0)
	end
	frame:SetLayer(100)
	frame.parent = parent
	frame.MouseDown = false

	function frame.MouseButtonDown(fButton)
		if (fButton == fLeft) then
			frame.MouseDown = true
			frame.Drag = false
			local mouseData = Inspect.Mouse()
			frame.xDiff = frame.parent:GetLeft() - mouseData.x
			frame.yDiff = frame.parent:GetTop() - mouseData.y
			if frame.parent.Event.RightDown then frame.parent.Event.RightDown() end
		end
	end

	function frame.Event:LeftDown()
		self.MouseButtonDown(true)
		if self.parent.Event.LeftDown then self.parent.Event.LeftDown() end
	end

	function frame.Event:RightDown()
		self.MouseButtonDown(false)
		if self.parent.Event.RightDown then self.parent.Event.RightDown() end
	end
	
	function frame.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			self.Drag = true
			self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouseX + self.xDiff, mouseY + self.yDiff)
			AuctionExtensionUISettings[variable].x = mouseX + self.xDiff
			AuctionExtensionUISettings[variable].y = mouseY + self.yDiff
		end
		if self.parent.Event.MouseMove then self.parent.Event.MouseMove() end
	end
	
	function frame.Event:LeftUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:LeftUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:RightUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:RightUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:LeftClick()
		if not self.Drag and self.parent.Event.LeftClick then self.parent.Event.LeftClick() end
	end

	function frame.Event:MouseIn()
		if self.parent.Event.MouseIn then self.parent.Event.MouseIn() end
	end

	function frame.Event:MouseOut()
		if self.parent.Event.MouseOut then self.parent.Event.MouseOut() end
	end
	return frame
end

function AuctionExtension.EGUI.CreateRow(parent, id, callback)
	local frame = AuctionExtension.GUI.CreateFrame(parent, "TOPLEFT", 2, id * 40 - 38, 838, 39, 10)
	frame.iconborder = AuctionExtension.GUI.CreateTexture(frame, 2, 2, 35, 35, 2)
	frame.icon = AuctionExtension.GUI.CreateTexture(frame, 4, 4, 31, 31, 1)
	frame.text = AuctionExtension.GUI.CreateText(frame, "TOPLEFT", 45, 9)
	frame.stack = AuctionExtension.GUI.CreateText(frame, "BOTTOMRIGHT", 38, 0, nil, 5, 10, nil, "BOTTOMLEFT")
	frame.field = AuctionExtension.GUI.CreateTexture(frame, 500, 9, 189, 23, 10, "img/field.png")
	frame.create = AuctionExtension.GUI.CreateRiftButton(frame, "TOPLEFT", 700, 3, 10, "Create")
	frame.create.Event.LeftPress = function() callback(frame.data, frame.action, frame.fields[1].GetValue() * 10000 + frame.fields[2].GetValue() * 100 + frame.fields[3].GetValue()) end

	frame.action = 3
	frame.currency = {}
	frame.fields = {}
	for i = 1, 3 do
		frame.currency[i] = AuctionExtension.EGUI.CreateCoinField(frame, iif(i == 3, 360, 250), iif(i == 3, -43, -15) + i * 19)
		frame.fields[i] = AuctionExtension.GUI.CreateNumericField(frame, "TOPLEFT", 505 + iif(i == 2, 93, iif(i == 3, 145, 0)), 11, iif(i == 1, 57, 20), 19, 20, iif(i == 1, 7, 2))
	end

	function frame.SetEnabled(state)
		state = iif(frame.action == 1, true, iif(frame.action < 3, state, false))
		local fieldState = iif(frame.action == 1, true, iif(frame.action == 0 and state, true, false))
		frame.field:SetVisible(fieldState)
		frame.fields[1]:SetVisible(fieldState)
		frame.fields[2]:SetVisible(fieldState)
		frame.fields[3]:SetVisible(fieldState)
		frame.create:SetVisible(state)
		frame.enabled = state
	end

	function frame.Update(data, source)
		if data then
			frame.data = data
			frame.Show()
			local rarity = data.rarity or "common"
			local color = AuctionExtension.Colors[rarity]
			frame.text:SetText(data.name)
			frame.text:SetFontColor(color[1] / 255, color[2] / 255, color[3] / 255, 1)
			frame.icon:SetTexture("Rift", data.icon)
			frame.iconborder:SetTexture(AuctionExtensionAddon.identifier, "img/border-" .. rarity .. ".png")
			frame.stack:SetText(iif(data.stack == 1, "", "" .. ifnull(data.stack, 0)))

			local aData = AuctionExtension.AuctionList[data.name] or { minPrice = 0, maxPrice = 0, ownPrice = 0 }
			frame.currency[1].SetValue(aData.minPrice)
			frame.currency[2].SetValue(aData.maxPrice)
			frame.currency[3].SetValue(aData.ownPrice)

			local proposal = iif(aData.minPrice == 0, AuctionExtension.PriceList[rarity].minimum, iif(aData.minPrice == aData.ownPrice, aData.ownPrice, math.min(AuctionExtension.PriceList[rarity].maximum, aData.minPrice + AuctionExtension.PriceList[rarity].delta)))
			local c1, c2, c3 = string.match("     " .. proposal, "(.+)(..)(..)") 
			frame.fields[1].SetValue(c1)
			frame.fields[2].SetValue(c2)
			frame.fields[3].SetValue(c3)

			frame.action = iif(source == 1, iif(aData.ownPrice > aData.minPrice, 2, 3), iif(proposal < AuctionExtension.DropPrice, 1, 0))
			frame.create:SetText(AuctionExtension.Buttons[frame.action + 1])
		end
		frame:SetVisible(data ~= nil)
	end

	function frame.Event:MouseIn() frame.hover = true frame.Show() Command.Tooltip(frame.data.id) end
	function frame.Event:MouseOut() frame.hover = false frame.Show() Command.Tooltip(nil) end
	function frame.Event:MouseMove() frame.Show() Command.Tooltip(frame.data.id) end
	function frame.Show() 
		frame:SetBackgroundColor(1, 1, 1, 0.05 + iif(frame.hover, .1, 0)) 
	end

	return frame
end

function AuctionExtension.EGUI.CreateCoinField(parent, x, y)
	local frame = AuctionExtension.GUI.CreateFrame(parent, "TOPLEFT", x, y, 100, 19, 20)
	frame.coins = {}
	frame.values = {}
	for i = 1, 3 do
		frame.coins[i] = AuctionExtension.GUI.CreateTexture(frame, i * 30 + 15, 0, 12, 12, 1, "coins_" .. AuctionExtension.Coins[i] .. ".png.dds", nil, true)
		frame.values[i] = AuctionExtension.GUI.CreateText(frame, "TOPRIGHT", i * 30 - 54 + 70, -3, nil, 1, 11, nil, "TOPLEFT")
	end

	function frame.SetValue(value)
		frame:SetVisible(value ~= 0)
		if value > 0 then
			local c1, c2, c3 = string.match("     " .. value, "(.+)(..)(..)") 
			frame.coins[1]:SetVisible(value > 9999)
			frame.values[1]:SetText(iif(value > 9999, c1, ""))
			frame.coins[2]:SetVisible(value > 99)
			frame.values[2]:SetText(iif(value > 99, c2, ""))
			frame.coins[3]:SetVisible(value > 0)
			frame.values[3]:SetText(iif(value > 0, c3, ""))
		end
	end

	return frame
end