function AuctionExtension.GUI.CreateFrame(parent, align, x, y, w, h, layer, uiParent)
	local frame = UI.CreateFrame("Frame", "", iif(uiParent == nil, parent, uiParent))
	frame:SetPoint(align, parent, "TOPLEFT", x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	return frame
end

function AuctionExtension.GUI.CreateText(parent, align, x, y, text, layer, size, color, falign)
	local frame = UI.CreateFrame("Text", "", parent)
	frame:SetFontSize(size or 14)
	frame:SetPoint(align, parent, falign or align, x, y)
	if text then frame:SetText(text) end
	if layer then frame:SetLayer(layer) end
	if color then frame:SetFontColor(color[1], color[2], color[3], color[4]) end

	return frame
end

function AuctionExtension.GUI.CreateRiftButton(parent, align, x, y, layer, text)
	local frame = UI.CreateFrame("RiftButton", "", parent)
	frame:SetPoint(align, parent, align, x, y)
	if text == nil then
		frame:SetSkin("close")
	else
		frame:SetText(text)
	end
	frame:SetLayer(layer)
	return frame
end

function AuctionExtension.GUI.CreateMaskedLayer(parent, align, x, y, w, h, layer, color, callback)
	local frame = UI.CreateFrame("Mask", "", parent)
	frame:SetPoint(align, parent, align, x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	frame:SetBackgroundColor(color[1], color[2], color[3], color[4])

	function frame.Event.WheelBack() callback(20) end
	function frame.Event.WheelForward() callback(-20) end

	frame.layer = UI.CreateFrame("Frame", "", frame)
	frame.layer:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
	frame.layer:SetWidth(w)

	return frame
end

function AuctionExtension.GUI.CreateScrollbar(parent, align, x, y, w, h, layer, orientation, callback)
	local frame = UI.CreateFrame("RiftScrollbar", "", parent)
	frame:SetPoint(align, parent, align, x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	frame:SetOrientation(orientation)
	frame.Event.ScrollbarChange = callback

	function frame.SetScroll(length)
		frame.maximum = iif(length > 10, length - 10, 0) * 40
		frame:SetEnabled(length > 10)
		frame:SetRange(0, frame.maximum)
		frame:SetPosition(0)
	end
	return frame
end

function AuctionExtension.GUI.CreateButton(parent, x, y, w, h, layer, texture, layout, isSwitch)
	local frame = UI.CreateFrame("Texture", "", parent)
	if x and y then frame:SetPoint(ifnull(layout, "TOPLEFT"), parent, ifnull(layout, "TOPLEFT"), x, y) end
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-normal.png")
	frame.state = false

	function frame.Event:LeftDown()
		frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-down.png")
	end

	function frame.Event:MouseIn()
		frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "") .. "hover.png")
	end

	function frame.Event:MouseMove()
		frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "") .. "hover.png")
	end

	function frame.Event:MouseOut()
		frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "normal") .. ".png")
	end

	function frame.SetState(flag)
		frame.state = flag
		frame:SetTexture(AuctionExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "normal") .. ".png")
	end
	return frame
end

function AuctionExtension.GUI.CreateCheckbox(parent, align, x, y, layer, text, callback, param, value, iconTexture)
	local frame = AuctionExtension.GUI.CreateFrame(parent, align, x, y, 100, 19, 0)

	frame.checkbox = UI.CreateFrame("RiftCheckbox", "", frame)
	frame.checkbox:SetPoint("TOPLEFT", frame, "TOPLEFT", 7, 2)
	frame.checkbox:SetLayer(layer)
	frame.checkbox:SetEnabled(true)
	frame.checkbox:SetChecked(ifnull(value, false))

	if iconTexture ~= nil then
		AuctionExtension.GUI.CreateSimpleTexture(frame, 23, 3, 15, 15, layer):SetTexture("Rift", iconTexture)
	end

	frame.text = UI.CreateFrame("Text", "", frame)
	frame.text:SetPoint("TOPLEFT", frame, "TOPLEFT", iif(iconTexture == nil, 23, 40), 0)
	frame.text:SetFontSize(14)
	frame.text:SetText(text)
	
	frame.checkbox.Event.CheckboxChange = function() callback(frame, param) end
	
	return frame
end

function AuctionExtension.GUI.CreateTexture(parent, x, y, w, h, layer, texture, layout, source)
	local frame = UI.CreateFrame("Texture", "", parent)
	frame:SetPoint(ifnull(layout, "TOPLEFT"), parent, ifnull(layout, "TOPLEFT"), x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	if texture then	frame:SetTexture(iif(source, "Rift", AuctionExtensionAddon.identifier), texture) end
	return frame
end

function AuctionExtension.GUI.CreateTileTexture(parent, marginTop, marginRight, marginBottom, marginLeft, w, h, layer, textureC, textureH, textureV)
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft, y = marginTop }}, layer, textureC, w, h)
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "TOPRIGHT", x = -marginRight, y = marginTop }}, layer, textureC, w, h)
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "BOTTOMLEFT", x = marginLeft, y = -marginBottom }}, layer, textureC, w, h)
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "BOTTOMRIGHT", x = -marginRight, y = -marginBottom }}, layer, textureC, w, h)
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft + w, y = marginTop }, { layout = "TOPRIGHT", x = - marginRight - w, y = marginTop }}, layer, textureH, nil, h, {{ layout = "TOPLEFT", x = 0, y = 0 }, { layout = "TOPRIGHT", x = 0, y = 0 }})
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "BOTTOMLEFT", x = marginLeft + w, y = -marginBottom }, { layout = "BOTTOMRIGHT", x = - marginRight - w, y = -marginBottom }}, layer, textureH, nil, h, {{ layout = "BOTTOMLEFT", x = 0, y = 0 }, { layout = "BOTTOMRIGHT", x = 0, y = 0 }})
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft, y = marginTop + h }, { layout = "BOTTOMLEFT", x = marginLeft, y = -marginBottom - h }}, layer, textureV, w, nil, {{ layout = "TOPLEFT", x = 0, y = 0 }, { layout = "BOTTOMLEFT", x = 0, y = 0 }})
	AuctionExtension.GUI.CreateMaskedTexture(AuctionExtension.AE.UI.Window, {{ layout = "TOPRIGHT", x = -marginRight, y = marginTop + h }, { layout = "BOTTOMRIGHT", x = -marginRight, y = -marginBottom - h }}, layer, textureV, w, nil, {{ layout = "TOPRIGHT", x = 0, y = 0 }, { layout = "BOTTOMRIGHT", x = 0, y = 0 }})
end

function AuctionExtension.GUI.CreateMaskedTexture(parent, layout, layer, texture, w, h, layoutTexture)
	local frame = UI.CreateFrame("Mask", "", parent)
	for idx, layoutItem in ipairs(layout) do
		frame:SetPoint(layoutItem.layout, parent, layoutItem.layout, layoutItem.x, layoutItem.y)
	end
	if w then frame:SetWidth(w) end
	if h then frame:SetHeight(h) end
	frame:SetLayer(layer)

	frame.texture = UI.CreateFrame("Texture", "", frame)
	frame.texture:SetTexture(AuctionExtensionAddon.identifier, texture)

	if layoutTexture == nil then layoutTexture = layout end
	for idx, layoutItem in ipairs(layoutTexture) do
		frame.texture:SetPoint(layoutItem.layout, frame, layoutItem.layout, 0, 0)
	end
	return frame
end

function AuctionExtension.GUI.CreateNumericField(parent, align, x, y, w, h, layer, length)
	local frame = UI.CreateFrame("RiftTextfield", "", parent)
	frame:SetPoint(align, parent, align, x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)

	function frame.Event:TextfieldChange()
		frame:SetText(((frame:GetText():gsub("[^%d]","")):sub(1, length):gsub("^0+", "")))
	end

	function frame.GetValue()
		return tonumber(frame:GetText()) or 0
	end

	function frame.SetValue(value)
		frame:SetText(((value:gsub("[^%d]","")):sub(1, length):gsub("^0+", "")))
	end

	return frame
end